<?php
if(!function_exists('shm_operate')) {
    /*
     * saves,fetches or destroys variable in shared memory
     * note! variable will occpupy more that twice memory of it's original size
     * 
     * $addr - integer-type memory segment address
     * $data - variable or object you want to save
     * $flush - enable/disable destroying of  current memory segment
     *	note!	variable($data) or it's referrence must be passed to determine it's original size. consider using supplied 'shm_flush' function instead.
     *		alternatively this function can be extended to support $size parameter (or even collecting array of array($addr=>$size) in separate shm segment)
     * returns true on success or false on failure
     *  note!	if $data=(NULL or 0 or FALSE) - it will allways return false even on success retrieval of variable.
     * 
     * author: johnrembo@netzond.com
     */
    function shm_operate($addr,$data=false,$flush=false) {
	$memsize = ( strlen( serialize( $data ) ) + 44 ) * 2;		//calculate memory segment size (most likely this formula is correct only for arrays and objects, but nevermind)
	$maddr=$addr;
	$sem_id = sem_get($addr);					//get semaphore id (second attemt to the same $addr will return different semaphore address)
	$result=false;
	
	if($sem_id) {
    	    if(sem_acquire($sem_id)) {					//acquire semaphore id (blocks until necessary if semaphore id is currently acquired by another proccess)
                $shm_id = shm_attach($maddr,$memsize);			//opens(or creates) shared memory segment. every access will result in different $shm_id
                if ($flush) {
                    $result=shm_remove($shm_id);			//removing shared-memory id will destroy segment's contents.
                }else{
                    if($data===false) {
                        $result=@shm_get_var($shm_id,$maddr);		//fetches a variable from sared memory
                    } else {
                        $result=shm_put_var($shm_id,$maddr,$data);	//saves variable in shared memory
                    }
                    if ($result) shm_detach($shm_id);			//release
                    else shm_remove($shm_id);				//there is no point reserving memory for NULL data
                }
    	    }
    	    sem_release($sem_id);					//unreserver resources
    	    sem_remove($sem_id);
	}
	return $result;
    }
}

if (!function_exists('shm_flush')) {
    /*
     * destorys variable (and removes shared memory segment)
     * $addr - integer-type memory segment address
     */
    function shm_flush($addr) {
	$obj=shm_operate($addr);                    //fetch object or variable to determine it's size
	return shm_operate($addr,$obj,true);        //clean memory segment
    }
}
?>
